//////////////////////////////////////////////////////////////////////////////
// k13client.h
// -------------------
// K13 window decoration for KDE
// -------------------
// Copyright (c) 2003 David Johnson
// Please see the header file for copyright and license information.
//////////////////////////////////////////////////////////////////////////////

#include <kconfig.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>
#include <kwin/workspace.h>
#include <kwin/options.h>

#include <qbitmap.h>
#include <qlayout.h>
#include <qpainter.h>

#include <kstddirs.h>
#include <kdebug.h>
#include "k13client.h"

using namespace KWinInternal;

// global constants

static const int FRAMESIZE       = 4;

//////////////////////////////////////////////////////////////////////////////
// K13Handler Class                                                     //
//////////////////////////////////////////////////////////////////////////////

bool K13Handler::initialized_              = false;
Qt::AlignmentFlags K13Handler::titlealign_ = Qt::AlignHCenter;

//////////////////////////////////////////////////////////////////////////////
// K13Handler()
// ----------------
// Constructor

K13Handler::K13Handler() { reset(); }

//////////////////////////////////////////////////////////////////////////////
// ~K13Handler()
// -----------------
// Destructor

K13Handler::~K13Handler() { initialized_ = false; }

//////////////////////////////////////////////////////////////////////////////
// reset()
// -------
// Reset the handler

void K13Handler::reset()
{
    // read in the configuration
    readConfig();

    // reset all clients
    initialized_ = true;
    Workspace::self()->slotResetAllClientsDelayed();
}

//////////////////////////////////////////////////////////////////////////////
// readConfig()
// ------------
// Read in the configuration file

void K13Handler::readConfig()
{
    // create a config object
    KConfig config("kwink13rc");
    config.setGroup("General");

    // grab settings
    QString value = config.readEntry("TitleAlignment", "AlignHCenter");
    if (value == "AlignLeft") titlealign_ = Qt::AlignLeft;
    else if (value == "AlignHCenter") titlealign_ = Qt::AlignHCenter;
    else if (value == "AlignRight") titlealign_ = Qt::AlignRight;
}

//////////////////////////////////////////////////////////////////////////////
// K13Button Class                                                      //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// K13Button()
// ---------------
// Constructor

K13Button::K13Button(Client *parent, const char *name, const QString &tip,
				ButtonType type, QPixmap *p1, QPixmap *p2, QPixmap *p3)
	: KWinButton(parent, name, tip), client_(parent), type_(type),
      deco_(p1), deco_off_(p1), deco_on_(p2), deco_click_(p3), lastmouse_(0)
{
    setBackgroundMode(NoBackground);
    //setFixedSize(BUTTONSIZE, BUTTONSIZE);
    if (deco_off_) setCurrentPixmap(deco_off_);
}

K13Button::~K13Button()
{
	/*if (deco_) delete deco_;
	if (deco_off_) delete deco_off_;
	if (deco_on_) delete deco_on_;
	if (deco_click_) delete deco_click_;*/
	kdDebug() << "K13Button destructor called" << endl;
}

//////////////////////////////////////////////////////////////////////////////
// setBitmap()
// -----------
// Set the button decoration

void K13Button::setCurrentPixmap(QPixmap *pix)
{
	if (!pix)
		return; // no bitmap, probably the menu button
	deco_ = pix;
	//deco_->setMask(*deco_);
	repaint(false);
}

//////////////////////////////////////////////////////////////////////////////
// sizeHint()
// ----------
// Return size hint

QSize K13Button::sizeHint() const
{
    return QSize(width(), height());
}

//////////////////////////////////////////////////////////////////////////////
// enterEvent()
// ------------
// Mouse has entered the button

void K13Button::enterEvent(QEvent *e)
{
    // if we wanted to do mouseovers, we would keep track of it here
    KWinButton::enterEvent(e);
}

//////////////////////////////////////////////////////////////////////////////
// leaveEvent()
// ------------
// Mouse has left the button

void K13Button::leaveEvent(QEvent *e)
{
    // if we wanted to do mouseovers, we would keep track of it here
    KWinButton::leaveEvent(e);
}

//////////////////////////////////////////////////////////////////////////////
// mousePressEvent()
// -----------------
// Button has been pressed

void K13Button::mousePressEvent(QMouseEvent* e)
{
    lastmouse_ = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    KWinButton::mousePressEvent(&me);
}

//////////////////////////////////////////////////////////////////////////////
// mouseReleaseEvent()
// -----------------
// Button has been released

void K13Button::mouseReleaseEvent(QMouseEvent* e)
{
    lastmouse_ = e->button();
    // pass on event after changing button to LeftButton
    QMouseEvent me(e->type(), e->pos(), e->globalPos(),
                   LeftButton, e->state());
    KWinButton::mouseReleaseEvent(&me);
}

//////////////////////////////////////////////////////////////////////////////
// drawButton()
// ------------
// Draw the button

void K13Button::drawButton(QPainter *painter)
{
	if (!K13Handler::initialized())
		return;

	//QColorGroup group;
	/*int dx, dy;

	if (type_ == ButtonMenu)
	{
		// paint a plain box with border
		//group = options->colorGroup(Options::ButtonBg, client_->isActive());
		//painter->fillRect(rect(), group.button());
		//painter->setPen(group.dark());
		kdDebug() << "proc drawButton Begin 1" << endl;
		painter->drawRect(rect());
		// we paint the mini icon (which is 16 pixels high)
		dx = (width() - 16) / 2;
		dy = (height() - 16) / 2;
		painter->drawPixmap(dx, dy, client_->miniIcon());
		kdDebug() << "proc drawButton Finish 1" << endl;
	}
	else
	{*/
		// otherwise we paint the deco
		//dx = (width() - DECOSIZE) / 2;
		//dy = (height() - DECOSIZE) / 2;
		//if (isDown()) { dx++; dy++; }
		//painter->setPen(group.dark());
		
		//kdDebug() << "proc drawButton Begin 2" << endl;
		if (deco_) painter->drawPixmap(0, 0, *deco_);
		//kdDebug() << "proc drawButton Step 2.1" << endl;
		//setPixmap(*deco_);
		setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
		setFixedSize(deco_->width(), deco_->height());
		//kdDebug() << "proc drawButton Step 2.2" << endl;
		show();
		//kdDebug() << "proc drawButton Finish 2" << endl;
	//}
}

//////////////////////////////////////////////////////////////////////////////
// K13Client Class                                                      //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// K13Client()
// ---------------
// Constructor

K13Client::K13Client(Workspace *ws, WId w, QWidget *parent, const char *name)
    : Client(ws, w, parent, "k13client", WResizeNoErase | WRepaintNoErase), titlebar_(0)
{
    // for flicker-free redraws
    setBackgroundMode(NoBackground);

	// setup layout
	QGridLayout *mainlayout = new QGridLayout(this, 3, 1, 0, -1, "mainLayout"); // 4x3 grid
	QHBoxLayout *titlelayout = new QHBoxLayout(this, 0, -1, "titleLayout");
	QHBoxLayout *innerlayout = new QHBoxLayout(this, 0, -1, "innerLayout");
	QVBoxLayout *buttonlayout = new QVBoxLayout(this, 0, -1, "buttonLayout");
	
   mainlayout->setResizeMode(QLayout::FreeResize);
   mainlayout->addRowSpacing(2, FRAMESIZE*2); // bottom border
	
	mainlayout->addLayout(titlelayout, 0, 0);
	mainlayout->addLayout(innerlayout, 1, 0);
	buttonlayout->setAlignment(Qt::AlignTop + Qt::AlignRight);
	innerlayout->addLayout(buttonlayout);
	innerlayout->addWidget(windowWrapper(), 10);

    // the window should stretch
    mainlayout->setRowStretch(1, 10);
    mainlayout->setColStretch(0, 10);
	// setup titlebar buttons
	for (int n = 0; n < ButtonTypeCount; n++)
	{
		button[n] = 0;
	}
	
	QString picturepath = locate ("data", "kwin/k13/win_l_1.png").replace("win_l_1.png", "");
	if (picturepath.isNull() || picturepath == "")
	{
		kdDebug() << "pics not found. problem with installation." << endl;
		picturepath = "/opt/kde/share/apps/kwin/k13/";
	}
	else
	{
		kdDebug() << "picture path found:" << picturepath << endl;
	}

	if ((!button[ButtonClose]) && isCloseable())
	{
		button[ButtonClose] = new K13Button(this, "close", i18n("Close"),
			ButtonClose, new QPixmap(picturepath + "win_l_1.png"),
			new QPixmap(picturepath + "win_l_2.png"), new QPixmap(picturepath + "win_l_3.png"));
		connect(button[ButtonClose], SIGNAL(clicked()),
				this, SLOT(closeWindow()));
		titlelayout->addWidget(button[ButtonClose]);
	}
	titlebar_ = 0;
	titlebar_ = addPixLabel(titlelayout, picturepath + "win_m_1.png", &titlebar_, 0, Qt::AlignLeft + Qt::AlignBottom);
	titlebar_ = addPixLabel(titlelayout, picturepath + "win_n_1.png", &titlebar_, 0, Qt::AlignLeft + Qt::AlignBottom);
	//titlebar_ = addPixLabel(titlelayout, picturepath + "win_o_1.png", &titlebar_, 0, Qt::AlignLeft + Qt::AlignBottom);
	//titlebar_ = addPixLabel(titlelayout, picturepath + "win_p_1.png", &titlebar_, 0, Qt::AlignLeft + Qt::AlignBottom);

	if ((!button[ButtonMin]) && isMinimizable())
	{
		button[ButtonMin] = new K13Button(this, "iconify", i18n("Minimize"),
			ButtonMin, new QPixmap(picturepath + "win_a_1.png"),
			new QPixmap(picturepath + "win_a_2.png"), new QPixmap(picturepath + "win_a_3.png"));
		connect(button[ButtonMin], SIGNAL(clicked()),
				this, SLOT(iconify()));
		buttonlayout->addWidget(button[ButtonMin]);
	}
	if (!button[ButtonMenu])
	{
		button[ButtonMenu] = new K13Button(this, "menu", i18n("Menu"),
			ButtonMenu, new QPixmap(picturepath + "win_b_1.png"),
			new QPixmap(picturepath + "win_b_2.png"), new QPixmap(picturepath + "win_b_3.png"));
		connect(button[ButtonMenu], SIGNAL(pressed()),
				this, SLOT(menuButtonPressed()));
		buttonlayout->addWidget(button[ButtonMenu]);
	}
	if (!button[ButtonSticky])
	{
		button[ButtonSticky] = new K13Button(this, "sticky", i18n("Sticky"),
			ButtonSticky, new QPixmap(picturepath + "win_c_1.png"),
			new QPixmap(picturepath + "win_c_2.png"), new QPixmap(picturepath + "win_c_3.png"));
		connect(button[ButtonSticky], SIGNAL(clicked()),
				this, SLOT(toggleSticky()));
		buttonlayout->addWidget(button[ButtonSticky]);
	}
                  
	addPixLabel(buttonlayout, picturepath + "win_d_1.png", 0, 0, Qt::AlignRight); //(QWidget*)buttonlayout->parent());
	addPixLabel(buttonlayout, picturepath + "win_e_1.png", 0, 0, Qt::AlignRight); //(QWidget*)buttonlayout->parent());
	addPixLabel(buttonlayout, picturepath + "win_f_1.png", 0, 0, Qt::AlignRight); //(QWidget*)buttonlayout->parent());
	
	/*if ((!button[ButtonHelp]) && providesContextHelp())
	{
		button[ButtonHelp] =
			new K13Button(this, "help", i18n("Help"),
								ButtonHelp, help_bits);
		connect(button[ButtonHelp], SIGNAL(clicked()),
				this, SLOT(contextHelp()));
		layout->addWidget(button[ButtonHelp]);
	}
                  

	if ((!button[ButtonMax]) && isMaximizable())
	{
		button[ButtonMax]  =
			new K13Button(this, "maximize", i18n("Maximize"),
								ButtonMax, max_bits);
		connect(button[ButtonMax], SIGNAL(clicked()),
				this, SLOT(maxButtonPressed()));
		layout->addWidget(button[ButtonMax]);
	}
                  
*/              
	//layout->addSpacing(FRAMESIZE); // Spacer item
	kdDebug() << "crash Final K13Client Konstructor Done" << endl;
	kdDebug() << "sss:" << picturepath << ":sss" << endl;
}

K13Client::~K13Client()
{
    for (int n=0; n<ButtonTypeCount; n++) {
        if (button[n]) delete button[n];
    }
}

QLabel* K13Client::addPixLabel(QBoxLayout *opLayout, QString spPixName, QLabel **ppLabel, QWidget* opLabel, int align)
{
	QLabel *l;
	if (ppLabel == 0)
	{
		ppLabel = &l;
	}
	*ppLabel = new QLabel(this); //(QWidget*)opLayout->parent());
	(*ppLabel)->setPixmap(QPixmap(spPixName));
	(*ppLabel)->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	(*ppLabel)->setFixedSize((*ppLabel)->pixmap()->width(), (*ppLabel)->pixmap()->height());
	(*ppLabel)->show();
	opLayout->addWidget(*ppLabel, 0, align);
	return *ppLabel;
}

//////////////////////////////////////////////////////////////////////////////
// paintEvent()
// ------------
// Repaint the window

void K13Client::paintEvent(QPaintEvent*)
{
    if (!K13Handler::initialized()) return;

    QPainter painter(this);

    QColorGroup group;

    // draw the titlebar
    QRect title(titlebar_->geometry());

    group = options->colorGroup(Options::TitleBar, isActive());
    painter.fillRect(title, group.background());
    painter.setPen(group.dark());
    painter.drawRect(title);

    // draw title text
    painter.setFont(options->font(isActive(), false));
    painter.setPen(options->color(Options::Font, isActive()));

    painter.drawText(title.x() + FRAMESIZE, title.y(),
                     title.width() - FRAMESIZE * 2, title.height(),
                     K13Handler::titleAlign() | AlignVCenter,
                     caption());

    // draw frame
    group = options->colorGroup(Options::Frame, isActive());

    // fill in the regions
    QRect frame(0, 0, width(), FRAMESIZE);
    painter.fillRect(frame, group.background());
    frame.setRect(0, 0, FRAMESIZE, height());
    painter.fillRect(frame, group.background());
    frame.setRect(0, height() - FRAMESIZE*2, width(), FRAMESIZE*2);
    painter.fillRect(frame, group.background());
    frame.setRect(width()-FRAMESIZE, 0, FRAMESIZE, height());
    painter.fillRect(frame, group.background());

    // outline the parts
    painter.setPen(group.dark());
    frame = rect();
    painter.drawRect(frame);
    frame.setRect(frame.x() + FRAMESIZE-1, frame.y() + FRAMESIZE-1,
                  frame.width() - FRAMESIZE*2 +2, 
                  frame.height() - FRAMESIZE*3 +2);
    painter.drawRect(frame);
}

//////////////////////////////////////////////////////////////////////////////
// resizeEvent()
// -------------
// Window is being resized

void K13Client::resizeEvent(QResizeEvent *e)
{
    Client::resizeEvent(e);

    if ((isVisibleToTLW()) && (!testWFlags(WStaticContents))) {
        QRegion region = rect();
        region.subtract(titlebar_->geometry());
	erase(region);
    }
}

//////////////////////////////////////////////////////////////////////////////
// captionChange()
// ---------------
// The title has changed

void K13Client::captionChange(const QString &)
{
    repaint(titlebar_->geometry(), false);
}

//////////////////////////////////////////////////////////////////////////////
// mouseCoubleClickEvent()
// -----------------------
// Doubleclick on title

void K13Client::mouseDoubleClickEvent(QMouseEvent *e)
{
    if (titlebar_->geometry().contains(e->pos()))
        workspace()->performWindowOperation(this,
            options->operationTitlebarDblClick());
}

//////////////////////////////////////////////////////////////////////////////
// mousePosition()
// ---------------
// Return logical mouse position

Client::MousePosition K13Client::mousePosition(const QPoint &point) const
{
    const int corner = 24;
    MousePosition pos;

    if (point.y() <= FRAMESIZE) {
        // inside top frame
        if (point.x() <= corner)                 pos = TopLeft;
        else if (point.x() >= (width()-corner))  pos = TopRight;
        else                                     pos = Top;
    } else if (point.y() >= (height()-FRAMESIZE*2)) {
        // inside handle
        if (point.x() <= corner)                 pos = BottomLeft;
        else if (point.x() >= (width()-corner))  pos = BottomRight;
        else                                     pos = Bottom;
    } else if (point.x() <= FRAMESIZE) {
        // on left frame
        if (point.y() <= corner)                 pos = TopLeft;
        else if (point.y() >= (height()-corner)) pos = BottomLeft;
        else                                     pos = Left;
    } else if (point.x() >= width()-FRAMESIZE) {
        // on right frame
        if (point.y() <= corner)                 pos = TopRight;
        else if (point.y() >= (height()-corner)) pos = BottomRight;
        else                                     pos = Right;
    } else {
        // inside the frame
        pos = Center;
    }
    return pos;
}

//////////////////////////////////////////////////////////////////////////////
// iconChange()
// ------------
// The title has changed

void K13Client::iconChange()
{
	if (button[ButtonMenu])
	{
		button[ButtonMenu]->setCurrentPixmap(0);
		button[ButtonMenu]->repaint(false);
	}
}

//////////////////////////////////////////////////////////////////////////////
// activeChange()
// --------------
// window active state has changed

void K13Client::activeChange(bool)
{
    for (int n=0; n<ButtonTypeCount; n++)
        if (button[n]) button[n]->repaint(false);
    repaint(false);
}

//////////////////////////////////////////////////////////////////////////////
// maximizeChange()
// ----------------
// Maximized state has changed

void K13Client::maximizeChange(bool m)
{
	if (m) return;
    /*if (button[ButtonMax]) {
        button[ButtonMax]->setCurrentPixmap(m ? deco : max_bits);
        button[ButtonMax]->setTipText(m ? i18n("Restore") : i18n("Maximize"));
    }*/
}

//////////////////////////////////////////////////////////////////////////////
// stickyChange()
// ----------------
// Sticky state has changed

void K13Client::stickyChange(bool s)
{
	if (s) return;
    /*if (button[ButtonSticky]) {
        button[ButtonSticky]->setCurrentPixmap(s ? stickydown_bits : sticky_bits);
        button[ButtonSticky]->setTipText(s ? i18n("Un-Sticky") : i18n("Sticky"));
    }*/
}

//////////////////////////////////////////////////////////////////////////////
// maxButtonPressed()
// -----------------
// Max button was pressed

void K13Client::maxButtonPressed()
{
    if (button[ButtonMax]) {
        switch (button[ButtonMax]->lastMousePress()) {
          case MidButton:   maximize(MaximizeVertical);   break;
          case RightButton: maximize(MaximizeHorizontal); break;
          default:          maximize();
        }
    }
}

//////////////////////////////////////////////////////////////////////////////
// menuButtonPressed()
// -------------------
// Menu button was pressed (popup the menu)

void K13Client::menuButtonPressed()
{
    if (button[ButtonMenu]) {
        QPoint pt(button[ButtonMenu]->rect().bottomLeft().x(),
                  button[ButtonMenu]->rect().bottomLeft().y());
        workspace()->showWindowMenu(button[ButtonMenu]->mapToGlobal(pt), this);
        button[ButtonMenu]->setDown(false);
    }
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

static K13Handler *handler;

extern "C"
{
    Client* allocate(Workspace *ws, WId w, int)
    {
        return new K13Client(ws, w);
    }

    void init()
    {
        handler = new K13Handler();
    }

    void reset()
    {
        handler->reset();
    }

    void deinit()
    {
        delete handler;
    }
}

#include "k13client.moc"
